/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include "dixie.h"

#include <quipu/attrvalue.h>
#include <quipu/ds_error.h>
#include <quipu/commonarg.h>
#include <quipu/add.h>
#include <quipu/dua.h>

#define NULL	0

#define ADD_END(x)	{ dn_free(thedn); \
			if (as) as_free(as); \
			return(x); }

extern int	debug;
extern int	bound;
extern int	stream_connected;
extern short	timelimit, sizelimit;

/* args for the read call */
struct ds_addentry_arg	add_arg = { default_common_args };
struct DSError		add_error;

/* ARGSUSED */
addop(bp, rp, rpsize, dontderef)
char	*bp, *rp;
int	rpsize;
char	dontderef;
{
	DN		thedn = NULLDN;
	short		i;
	Attr_Sequence	as, as_combine();

	add_arg.ada_common.ca_servicecontrol.svc_timelimit = timelimit;
	add_arg.ada_common.ca_servicecontrol.svc_sizelimit = sizelimit;
	if ( dontderef )
		add_arg.ada_common.ca_servicecontrol.svc_options |=
		    SVC_OPT_DONTDEREFERENCEALIAS;
	else
		add_arg.ada_common.ca_servicecontrol.svc_options &=
		    ~SVC_OPT_DONTDEREFERENCEALIAS;
	*rp = '\0';
	if ( debug & 2 ) printf("Add (%s)\n", bp);

	/* 
	 * First convert the dn given into a DN structure
	 */

	if ( *bp == '@') bp++;
	if ( (thedn = str2dn(bp)) == NULLDN ) {
		if ( debug > 1 ) printf("readops: str2dn (%s) failed\n",bp);
		return( DXI_BADDN );
	}
	add_arg.ada_object = thedn;

	/*
	 * Next we get the null-separated list of attributes for the entry
	 */

	bp += strlen( bp ) + 1;
	as = NULLATTR;
	while ( *bp ) {
		if ( (as = as_combine( as, bp )) == NULLATTR ) {
			if ( debug & 2 ) printf("as_combine (%s) failed\n", bp);
			ADD_END( DXI_BAD_ATTRVAL );
		}

		bp += strlen( bp ) + 1;
	}
	add_arg.ada_entry = as;

	/* do the actual add */
	for ( i = 0; i < 2; i++ ) {
		if ( ds_addentry( &add_arg, &add_error ) == DS_OK )
			break;
		if ( debug & 2 ) printf("ds_add failed\n");

		if ( add_error.dse_type == DSE_REFERRAL )
			if ( chase_referral( &add_error ) != OK )
				break;
			else
				continue;

		if ( add_error.dse_type != DSE_REMOTEERROR
		    || stream_connected )
			break;

		(void) nullbind();

		if ( bound == 0 )
			break;
	}

	if ( debug & 2 )
		printf("add successful - checking for errors\n");

	switch ( add_error.dse_type ) {
	case DSE_UPDATEERROR:
		if ( debug & 2 ) {
			printf("\tupdate error\n");
			error( &add_error );
		}
		if ( add_error.dse_un.dse_un_update.DSE_up_problem ==
		    DSE_UP_ALREADYEXISTS )
			ADD_END( DXI_EXISTS );
		ADD_END( DXI_ERROR );
	case DSE_REMOTEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &add_error );
		}
		ADD_END( DXI_DSA_DOWN );
	case DSE_REFERRAL:
		if ( debug & 2 ) {
			printf("\treferral error\n");
			error( &add_error );
		}
		ADD_END( DXI_REFER_ERROR );
	case DSE_NAMEERROR:
		if ( debug & 2 ) {
			printf("\tname error\n");
			error( &add_error );
		}
		ADD_END( DXI_NAME_ERROR );
	case DSE_SERVICEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &add_error );
		}
		if (add_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_TIMELIMITEXCEEDED &&
		    add_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_ADMINLIMITEXCEEDED)
			ADD_END( DXI_ERROR )
		break;
	case DSE_NOERROR:
		if ( debug & 2 ) printf("\tno error\n");
		break;
	default:
		if ( debug & 2 ) {
			printf("\tsome unknown error (%d)\n");
			error( &add_error );
		}
		ADD_END( DXI_ERROR );
	}

	ADD_END( DXI_SUCCESS );
}
