/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include "dixie.h"

#include <quipu/attrvalue.h>
#include <quipu/ds_error.h>
#include <quipu/commonarg.h>
#include <quipu/modifyrdn.h>
#include <quipu/dua.h>

#define NULL	0

#define MODRDN_END(x)	{ dn_free(olddn); \
			rdn_free(newrdn); \
			return(x); }

extern int	debug;
extern int	bound;
extern int	stream_connected;
extern short	timelimit, sizelimit;

/* args for the modrdn call */
struct ds_modifyrdn_arg	modrdn_arg = { default_common_args };
struct DSError		modrdn_error;

/* ARGSUSED */
modrdnop(bp, rp, rpsize, dontderef)
char	*bp, *rp;
int	rpsize;
char	dontderef;
{
	DN		olddn = NULLDN;
	RDN		newrdn = NULLRDN;
	int		i;

	modrdn_arg.mra_common.ca_servicecontrol.svc_timelimit = timelimit;
	modrdn_arg.mra_common.ca_servicecontrol.svc_sizelimit = sizelimit;
	if ( dontderef )
		modrdn_arg.mra_common.ca_servicecontrol.svc_options |=
		    SVC_OPT_DONTDEREFERENCEALIAS;
	else
		modrdn_arg.mra_common.ca_servicecontrol.svc_options &=
		    ~SVC_OPT_DONTDEREFERENCEALIAS;
	*rp = '\0';
	if ( debug & 2 ) printf("Modrdn\n");

	/* 
	 * First convert the dn given into a DN structure
	 */

	if ( *bp == '@') bp++;
	if ( debug & 2 ) printf("\told: (%s)\n", bp);
	if ( (olddn = str2dn( bp )) == NULLDN ) {
		if ( debug > 1 ) printf("modrdnop: str2dn (%s) failed\n",bp);
		return( DXI_BADDN );
	}
	modrdn_arg.mra_object = olddn;

	bp += strlen( bp ) + 1;
	if ( debug & 2 ) printf("\tnew: (%s)\n", bp);
	if ( (newrdn = str2rdn( bp )) == NULLRDN ) {
		if ( debug > 1 ) printf("modrdnop: str2rdn (%s) failed\n",bp);
		dn_free( olddn );
		return( DXI_BADDN );
	}
	modrdn_arg.mra_newrdn = newrdn;
	modrdn_arg.deleterdn = TRUE;

	/* do the actual modrdn */
	for ( i = 0; i < 2; i++ ) {
		if ( ds_modifyrdn( &modrdn_arg, &modrdn_error ) == DS_OK )
			break;
		if ( debug & 2 ) printf("ds_modifyrdn failed\n");

		if ( modrdn_error.dse_type == DSE_REFERRAL )
			if ( chase_referral( &modrdn_error ) != OK )
				break;
			else
				continue;

		if ( modrdn_error.dse_type != DSE_REMOTEERROR
		    || stream_connected )
			break;

		(void) nullbind();

		if ( bound == 0 )
			break;
	}

	if ( debug & 2 )
		printf("modrdn successful - checking for errors\n");

	switch ( modrdn_error.dse_type ) {
	case DSE_REMOTEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &modrdn_error );
		}
		MODRDN_END( DXI_DSA_DOWN );
	case DSE_REFERRAL:
		if ( debug & 2 ) {
			printf("\treferral error\n");
			error( &modrdn_error );
		}
		MODRDN_END( DXI_REFER_ERROR );
	case DSE_NAMEERROR:
		if ( debug & 2 ) {
			printf("\tname error\n");
			error( &modrdn_error );
		}
		MODRDN_END( DXI_NAME_ERROR );
	case DSE_SERVICEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &modrdn_error );
		}
		if (modrdn_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_TIMELIMITEXCEEDED &&
		    modrdn_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_ADMINLIMITEXCEEDED)
			MODRDN_END( DXI_ERROR )
		break;
	case DSE_NOERROR:
		if ( debug & 2 ) printf("\tno error\n");
		break;
	default:
		if ( debug & 2 ) {
			printf("\tsome unknown error (%d)\n");
			error( &modrdn_error );
		}
		MODRDN_END( DXI_ERROR );
	}

	MODRDN_END( DXI_SUCCESS );
}
